@@ -22,23 +22,24 @@
 //
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
-#include "csscopeconfdata.h"
-#include "globals.h"
-#include "plugin.h"
-#include <wx/app.h>
-#include <wx/log.h>
-#include "cscopetab.h"
-#include "drawingutils.h"
-#include "cscopedbbuilderthread.h"
-#include "imanager.h"
-#include "fileextmanager.h"
-#include "workspace.h"
 #include "bitmap_loader.h"
-#include <wx/treectrl.h>
-#include <wx/imaglist.h>
-#include <set>
+#include "cscopedbbuilderthread.h"
+#include "cscopetab.h"
+#include "csscopeconfdata.h"
 #include "drawingutils.h"
 #include "event_notifier.h"
+#include "file_logger.h"
+#include "fileextmanager.h"
+#include "globals.h"
+#include "imanager.h"
+#include "plugin.h"
+#include "workspace.h"
+#include <set>
+#include <wx/app.h>
+#include <wx/imaglist.h>
+#include <wx/log.h>
+#include <wx/treectrl.h>
+#include <clFileSystemWorkspace.hpp>
 
 CscopeTab::CscopeTab(wxWindow* parent, IManager* mgr)
     : CscopeTabBase(parent)
@@ -46,14 +47,13 @@ CscopeTab::CscopeTab(wxWindow* parent, IManager* mgr)
     , m_mgr(mgr)
 {
     m_styler.Reset(new clFindResultsStyler(m_stc));
-    m_bitmaps = clGetManager()->GetStdIcons()->MakeStandardMimeMap();
 
     CScopeConfData data;
     m_mgr->GetConfigTool()->ReadObject(wxT("CscopeSettings"), &data);
 
     const wxString SearchScope[] = { wxTRANSLATE("Entire Workspace"), wxTRANSLATE("Active Project") };
-    m_stringManager.AddStrings(
-        sizeof(SearchScope) / sizeof(wxString), SearchScope, data.GetScanScope(), m_choiceSearchScope);
+    m_stringManager.AddStrings(sizeof(SearchScope) / sizeof(wxString), SearchScope, data.GetScanScope(),
+                               m_choiceSearchScope);
 
     wxFont defFont = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
     m_font = wxFont(defFont.GetPointSize(), wxFONTFAMILY_TELETYPE, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
@@ -68,8 +68,8 @@ CscopeTab::CscopeTab(wxWindow* parent, IManager* mgr)
 
 CscopeTab::~CscopeTab()
 {
-    EventNotifier::Get()->Disconnect(
-        wxEVT_CL_THEME_CHANGED, wxCommandEventHandler(CscopeTab::OnThemeChanged), NULL, this);
+    EventNotifier::Get()->Disconnect(wxEVT_CL_THEME_CHANGED, wxCommandEventHandler(CscopeTab::OnThemeChanged), NULL,
+                                     this);
 }
 
 void CscopeTab::Clear()
@@ -134,9 +134,7 @@ void CscopeTab::FreeTable()
 
 void CscopeTab::SetMessage(const wxString& msg, int percent)
 {
-    if(m_mgr->GetStatusBar()) {
-        m_mgr->GetStatusBar()->SetMessage(msg, 3);
-    }
+    if(m_mgr->GetStatusBar()) { m_mgr->GetStatusBar()->SetMessage(msg, 3); }
     m_gauge->SetValue(percent);
 }
 
@@ -176,7 +174,7 @@ void CscopeTab::OnCreateDB(wxCommandEvent& e)
 void CscopeTab::OnWorkspaceOpenUI(wxUpdateUIEvent& e)
 {
     CHECK_CL_SHUTDOWN();
-    e.Enable(m_mgr->IsWorkspaceOpen());
+    e.Enable(IsWorkspaceOpen());
 }
 
 void CscopeTab::OnThemeChanged(wxCommandEvent& e)
@@ -185,16 +183,6 @@ void CscopeTab::OnThemeChanged(wxCommandEvent& e)
     m_styler->SetStyles(m_stc);
 }
 
-wxBitmap CscopeTab::GetBitmap(const wxString& filename) const
-{
-    wxFileName fn(filename);
-    FileExtManager::FileType type = FileExtManager::GetType(filename);
-    if(m_bitmaps.count(type) == 0) {
-        type = FileExtManager::TypeText;
-    }
-    return m_bitmaps.find(type)->second;
-}
-
 void CscopeTab::ClearText()
 {
     m_stc->SetEditable(true);
@@ -219,8 +207,8 @@ void CscopeTab::AddFile(const wxString& filename)
 
 void CscopeTab::OnHotspotClicked(wxStyledTextEvent& e)
 {
-    CHECK_PTR_RET(clCxxWorkspaceST::Get()->IsOpen());
-
+    if(!IsWorkspaceOpen()) { return; }
+    
     int clickedLine;
     int style = m_styler->HitTest(e, clickedLine);
     if(style == clFindResultsStyler::LEX_FIF_FILE || style == clFindResultsStyler::LEX_FIF_HEADER) {
@@ -230,13 +218,42 @@ void CscopeTab::OnHotspotClicked(wxStyledTextEvent& e)
         // Open the match
         std::map<int, CscopeEntryData>::const_iterator iter = m_matchesInStc.find(clickedLine);
         if(iter != m_matchesInStc.end()) {
-            wxString wsp_path = clCxxWorkspaceST::Get()->GetPrivateFolder();
+            wxString wsp_path = GetWorkingDirectory();
             wxFileName fn(iter->second.GetFile());
             if(!fn.MakeAbsolute(wsp_path)) {
-                wxLogMessage(wxT("CScope: failed to convert file to absolute path"));
+                clLogMessage(wxT("CScope: failed to convert file to absolute path"));
                 return;
             }
             m_mgr->OpenFile(fn.GetFullPath(), "", iter->second.GetLine() - 1);
+
+            // In theory this isn't needed as it happened in OpenFile()
+            // In practice there's a timing issue: if the file needs to be loaded,
+            // the CenterLine() call arrives too soon. So repeat it here, delayed.
+            CallAfter(&CscopeTab::CenterEditorLine, iter->second.GetLine() - 1);
         }
     }
 }
+
+void CscopeTab::CenterEditorLine(int lineno)
+{
+    IEditor* editor = m_mgr->GetActiveEditor();
+    if(editor) { editor->CenterLine(lineno); }
+}
+
+wxString CscopeTab::GetWorkingDirectory() const
+{
+    if(!IsWorkspaceOpen()) { return wxEmptyString; }
+
+    if(clFileSystemWorkspace::Get().IsOpen()) {
+        wxFileName fn = clFileSystemWorkspace::Get().GetFileName();
+        fn.AppendDir(".codelite");
+        return fn.GetPath();
+    } else {
+        return clCxxWorkspaceST::Get()->GetPrivateFolder();
+    }
+}
+
+bool CscopeTab::IsWorkspaceOpen() const
+{
+    return clFileSystemWorkspace::Get().IsOpen() || clCxxWorkspaceST::Get()->IsOpen();
+}
